#
#   5B - Telephone
#

from pygame import Rect
from pygame.locals import *
from part import Part
from albow.resource import get_image, get_sound
from albow.image_array import get_image_array
from vector_algebra import Vector2

IDLE = 'idle'
WINDING = 'winding'
UNWINDING = 'unwinding'

def get_dialling_sounds():
	return [get_sound("phone/dial%s.ogg" % i)
		for i in xrange(10)]

def get_hook_sounds():
	return [get_sound("phone/%shook.ogg" % x)
		for x in ("on", "off")]

class Telephone(Part):

	color_names = ["Green", "Red"]
	nc = len(color_names)
	body_images = get_image_array("parts/phone.png", nc, border = 2)
	handset_images = get_image_array("parts/handset.png", nc, border = 2)
	dial_images = get_image_array("parts/dial.png", (2, 13), border = 2)
	terminals_image = get_image("parts/phone_terminals.png")
	dialling_sounds = get_dialling_sounds()
	hook_sounds = get_hook_sounds()
	onhook_sound = get_sound("phone/onhook.ogg")
	offhook_sound = get_sound("phone/offhook.ogg")
	
	size = (70, 90)
	terminal_points = [(-5, 40), (5, 40)]
	handset_offsets = [(0, -12), (0, -32)] # onhook, offhook
	body_offset = Vector2(0, 10)
	dial_offset = (0, 10)
	terminals_offset = (0, 40)
	
	def __init__(self):
		self.initially_off_hook = False
		self.off_hook = False
		self.dial_phase = 0
		Part.__init__(self)
	
	def init_from_catalog(self, color_no):
		self.color_no = color_no
		
	def reset(self):
		self.state = IDLE
		self.off_hook = self.initially_off_hook
		self.dial_phase = 0
		self.pulser_open = False
	
	def draw(self, surf, t):
		gc = t * self.position
		cn = self.color_no
		def b(image, offset):
			r = image.get_rect()
			r.center = (gc + Vector2(offset)).coords
			surf.blit(image, r)
		b(self.handset_images[cn], self.handset_offsets[self.off_hook])
		b(self.body_images[cn], self.body_offset)
		b(self.dial_images[self.dial_image_index()], self.dial_offset)
		b(self.terminals_image, self.terminals_offset)
	
	def dial_image_index(self):
		phase = min(self.dial_phase, 25)
		row = phase & 1
		col = phase >> 1
		return row, col

	def get_oriented_rect(self):
		r = Rect(((0, 0), self.size))
		r.center = (0, 0)
		return r

	def get_params(self, params):
		params.append(self.color_no)
		params.append(self.initially_off_hook)

	def read_args(self, r):
		Part.read_args(self, r)
		self.color_no = r.get_arg(int)
		self.initially_off_hook = r.get_arg(bool, False)

	def selectable_at_play_time(self):
		return self.color_no == 0

	def key_down(self, e):
		key = e.key
		c = e.unicode
		if "0" <= c <= "9":
			self.dial_digit(c)
			return True
		elif key == K_h:
			self.set_off_hook(not self.off_hook)
			return True
		
	def set_off_hook(self, state):
		if self.off_hook <> state:
			self.off_hook = state
			self.hook_sounds[self.off_hook].play()

	def dial_digit(self, c):
		n = ord(c) - ord("0")
		print "telephone: dial", n ###
		if self.state == IDLE:
			self.dialling_number = n
			self.windup_limit = 30 - 2 * n
			self.pulse_delay = 9
			self.state = WINDING

	def begin_frame(self):
		#self.dump_node_states() ###
		state = self.state
		if state == WINDING:
			if self.dial_phase == 0:
				self.dialling_sounds[self.dialling_number].play()
			self.dial_phase += 2
			if self.dial_phase >= self.windup_limit:
				self.state = UNWINDING
		elif state == UNWINDING:
			if self.pulse_delay > 0:
				self.pulse_delay -= 1
			elif self.dial_phase >= 2:
				self.pulser_open = self.dial_phase & 1
				#print "telephone: pulsing: phase =", self.dial_phase, "open =", self.pulser_open ###
			if self.dial_phase > 0:
				self.dial_phase -= 1
			else:
				self.state = IDLE

	def node_changed(self, i):
		if self.off_hook and not self.pulser_open:
			self.propagate(0, 1)

	def is_dialling(self):
		return self.state <> IDLE

	def edit_params(self):
		self.initially_off_hook = not self.initially_off_hook
		self.reset()
		return True
