#
#   5B - Villain action script executor
#

import sys

WAIT_DEADLINE = 0
RUN_SCRIPT = 1
HALT = 2
WAIT_START_DIAL = 3
WAIT_END_DIAL = 4
WAIT_TIME = 5
WAIT_HALT = 6

halt_delay = 500 # ms

from settings import milliseconds_to_frames

class Tyger(object):
	#  level   Level
	#  state   int
	#  script  [str]
	#  lineno  int
	#  charno  int
	#  phone   Telephone
	#  timeout int

	def __init__(self, level):
		self.level = level
		self.script = []
	
	def reset(self):
		self.state = WAIT_DEADLINE
		self.lineno = 0
		self.charno = 0
		self.phone = None
		self.timeout = 0
	
	def get_script(self):
		return ";".join(self.script)
	
	def set_script(self, x):
		if x:
			l = x.split(";")
		else:
			l = []
		self.script = l
	
	def read_from_file(self, r):
		if r.looking_at("Script"):
			self.set_script(r.get_arg(str))
			r.next_line()
	
	def write_to_file(self, f):
		f.write("Script [%r]\n" % str(self.get_script()))
	
	def deadline_reached(self):
		if self.state == WAIT_DEADLINE:
			self.phone = self.level.find_a_red_phone()
			if self.phone:
				self.state = RUN_SCRIPT
			else:
				sys.stderr.write("No red phone found\n")
				self.state = HALT
	
	def begin_frame(self):
		state = self.state
		if state == RUN_SCRIPT:
			self.run_script()
		elif state == WAIT_START_DIAL:
			if self.phone_is_dialling():
				self.state = WAIT_END_DIAL
		elif state == WAIT_END_DIAL:
			if not self.phone_is_dialling():
				self.state = RUN_SCRIPT
		elif state == WAIT_TIME:
			self.timeout -= 1
			if self.timeout <= 0:
				self.state = RUN_SCRIPT
		elif state == WAIT_HALT:
			self.timeout -= 1
			if self.timeout <= 0:
				self.state = HALT
				self.level.script_finished()
	
	def get_phone(self):
		return self.phone
	
	def phone_is_dialling(self):
		phone = self.get_phone()
		return phone and phone.is_dialling()
	
	def lift_handset(self):
		phone = self.get_phone()
		if phone:
			phone.set_off_hook(True)
	
	def hang_up(self):
		phone = self.get_phone()
		if phone:
			phone.set_off_hook(False)
	
	def dial_digit(self, c):
		if "0" <= c <= "9":
			phone = self.get_phone()
			if phone:
				phone.dial_digit(c)

	def run_script(self):
		script = self.script
		lineno = self.lineno
		charno = self.charno
		if lineno >= len(script):
			self.state = WAIT_HALT
			self.timeout = milliseconds_to_frames(halt_delay)
			return
		line = script[lineno]
		action = line[:1]
		if not action:
			self.lineno += 1
			self.charno = 0
		elif charno == 0:
			if action == "L":
				self.lift_handset()
				self.lineno += 1
			elif action == "H":
				self.hang_up()
				self.lineno += 1
			elif action == "D":
				self.charno = 1
			elif action == "W":
				try:
					ms = int(line[1:])
				except ValueError:
					ms = 0
				self.timeout = milliseconds_to_frames(ms)
				self.state = WAIT_TIME
				self.lineno += 1
		else:
			if action == "D":
				if charno >= len(line):
					self.state = RUN_SCRIPT
					self.lineno += 1
					self.charno = 0
				else:
					self.dial_digit(line[charno])
					self.state = WAIT_START_DIAL
					self.charno += 1

	def is_active(self):
		return self.state <> WAIT_DEADLINE and self.phone is not None

	def give_up(self):
		self.state = HALT

	def finished(self):
		return self.state == HALT
	